@@ -86,7 +86,7 @@
 #    define IUTEST_PLATFORM             "LINUX"
 #  endif
 #elif defined(__native_client__)
-#  define IUTEST_OS_NACL                1
+#  define IUTEST_OS_NACL                1   //!< @deprecated native client to eol
 #  define IUTEST_PLATFORM               "Google Native Client"
 #elif defined(__AVR32__) || defined(__avr32__)
 #  define IUTEST_OS_AVR32               1
@@ -100,6 +100,49 @@
 #  include <android/api-level.h>
 #endif
 
+// c++17
+
+//! is c++17 compiler
+#if !defined(IUTEST_HAS_CXX17)
+#  if (defined(__cplusplus) && __cplusplus >= 201406L)
+#    define IUTEST_HAS_CXX17        1
+#  elif defined(_MSVC_LANG) && _MSVC_LANG > 201402
+#    define IUTEST_HAS_CXX17        1
+#  endif
+#endif
+
+#if !defined(IUTEST_HAS_CXX17)
+#  define IUTEST_HAS_CXX17          0
+#endif
+
+//! inline variable
+#if !defined(IUTEST_HAS_INLINE_VARIABLE)
+#if defined(__clang__)
+#  if (__clang_major__ > 3 || (__clang_major__ == 3 && __clang_minor__ >= 9)) && IUTEST_HAS_CXX17
+#    define IUTEST_HAS_INLINE_VARIABLE      1
+#  endif
+#endif
+#endif
+
+#if !defined(IUTEST_HAS_INLINE_VARIABLE)
+#  define IUTEST_HAS_INLINE_VARIABLE        0
+#endif
+
+//! constexpr if
+#if !defined(IUTEST_HAS_CONSTEXPR_IF)
+#  if   defined(__cpp_if_constexpr) && __cpp_if_constexpr >= 201606
+#    define IUTEST_HAS_CONSTEXPR_IF         1
+#  elif defined(_MSC_VER)
+#    if _MSC_VER >= 1910 && IUTEST_HAS_CXX17
+#      define IUTEST_HAS_CONSTEXPR_IF       1
+#    endif
+#  endif
+#endif
+
+#if !defined(IUTEST_HAS_CONSTEXPR_IF)
+#  define IUTEST_HAS_CONSTEXPR_IF           0
+#endif
+
 // c++11
 
 //! is c++11 compiler
@@ -825,7 +868,7 @@
 
 //! explicit instantiation access checking (static member function)
 #if !defined(IUTEST_EXPLICIT_INSTANTIATION_ACCESS_PRIVATE_STATIC_MEMBER_FUNCTION)
-#  if defined(_MSC_VER)
+#  if defined(_MSC_VER) && (_MSC_VER < 1910)
 // Visual Studio では、private な static メンバー関数に explicit instantiation でもアクセスできない
 #    define IUTEST_EXPLICIT_INSTANTIATION_ACCESS_PRIVATE_STATIC_MEMBER_FUNCTION 0
 #  else
