@@ -6,7 +6,7 @@
  *
  * @author      t.shirayanagi
  * @par         copyright
- * Copyright (C) 2012-2016, Takazumi Shirayanagi\n
+ * Copyright (C) 2012-2017, Takazumi Shirayanagi\n
  * This software is released under the new BSD License,
  * see LICENSE
 */
@@ -726,6 +726,8 @@
 #  define IUTEST_LIB_TOOLSET    "vc120"
 #elif _MSC_VER == 1900
 #  define IUTEST_LIB_TOOLSET    "vc140"
+#elif _MSC_VER == 1910
+#  define IUTEST_LIB_TOOLSET    "vc141"
 #else
 #  error unknown _MSC_VER.
 #endif
