@@ -6,7 +6,7 @@
  *
  * @author      t.shirayanagi
  * @par         copyright
- * Copyright (C) 2011-2016, Takazumi Shirayanagi\n
+ * Copyright (C) 2011-2017, Takazumi Shirayanagi\n
  * This software is released under the new BSD License,
  * see LICENSE
 */
@@ -32,6 +32,7 @@
  *          also_run_disabled_tests (bool)\n
  *          break_on_failure (bool)\n
  *          throw_on_failure (bool)\n
+ *          warning_into_error (bool)\n
  *          catch_exceptions (bool)\n
  *          catch_exceptions_each   (bool)\n
  *          catch_exceptions_global (bool)\n
@@ -136,9 +137,6 @@ public:
      * @brief   フラグ
     */
     enum Kind
-#if IUTEST_HAS_STRONG_ENUMS
-        : unsigned int
-#endif
     {
         SHUFFLE_TESTS           = 0x00000001,   //!< シャッフルテスト
         RUN_DISABLED_TESTS      = 0x00000002,   //!< DISABLED テストも実行
@@ -146,6 +144,7 @@ public:
 
         BREAK_ON_FAILURE        = 0x00000010,   //!< テスト失敗時にブレーク
         THROW_ON_FAILURE        = 0x00000040,   //!< 致命的な失敗時に throw する
+        WARNING_INTO_ERROR      = 0x00000080,   //!< 警告をエラーとする
 
         CONSOLE_COLOR_ON        = 0x00000100,   //!< 色つき出力ON
         CONSOLE_COLOR_OFF       = 0x00000200,   //!< 色つき出力OFF
@@ -159,17 +158,18 @@ public:
         CATCH_EXCEPTION_GLOBAL  = 0x00020000,   //!< 例外を catch する(UnitTest)
         CATCH_EXCEPTION         = 0x00030000,   //!< 例外を catch する
 
-        SHOW_TESTS_LIST         = 0x02000000,   //!< テストのリスト表示
-        SHOW_TESTS_LIST_WITH_WHERE
-                                = 0x04000000,   //!< テストのリスト表示(with where)
+        SHOW_HELP               = 0x01000000,   //!< ヘルプ表示
+        SHOW_VERSION            = 0x02000000,   //!< バージョン表示
+        SHOW_FEATURE            = 0x04000000,   //!< 機能の出力
+        SHOW_SPEC               = 0x08000000,   //!< Specの出力
 
-        SHOW_HELP               = 0x10000000,   //!< ヘルプ表示
-        SHOW_VERSION            = 0x20000000,   //!< バージョン表示
-        SHOW_FEATURE            = 0x40000000,   //!< 機能の出力
-        SHOW_SPEC               = 0x80000000,   //!< Specの出力
-        SHOW_INFO_MASK          = 0xF0000000,   //!< 情報表示系マスク
-        SHOW_MASK               = 0xFF000000,   //!< 表示系マスク
-        MASK                    = 0xFFFFFFFF,   //!< マスク
+        SHOW_TESTS_LIST         = 0x20000000,   //!< テストのリスト表示
+        SHOW_TESTS_LIST_WITH_WHERE
+                                = 0x40000000,   //!< テストのリスト表示(with where)
+
+        SHOW_INFO_MASK          = 0x0F000000,   //!< 情報表示系マスク
+        SHOW_MASK               = 0x7F000000,   //!< 表示系マスク
+        MASK                    = 0x7FFFFFFF,   //!< マスク
 
         //! デフォルト
 #if defined(_MSC_VER)
@@ -212,8 +212,8 @@ private:
         typedef Fragment<KIND> _Myt;
     public:
         Fragment() IUTEST_CXX_NOEXCEPT_SPEC {}
-        Fragment(bool f) { SetFlag(KIND, f ? -1 : ~KIND); } // NOLINT
-        _Myt& operator = (bool f) { SetFlag(KIND, f ? -1 : ~KIND); return *this; }
+        Fragment(bool enabled) { SetFlag(KIND, enabled ? -1 : ~KIND); } // NOLINT
+        _Myt& operator = (bool enabled) { SetFlag(KIND, enabled ? -1 : ~KIND); return *this; }
         operator bool() const { return IsEnableFlag(KIND); }
     };
 
@@ -246,6 +246,7 @@ public:
     typedef TestFlag::Fragment<TestFlag::BREAK_ON_FAILURE>      break_on_failure;
     typedef TestFlag::Fragment<TestFlag::CATCH_EXCEPTION>       catch_exceptions;
     typedef TestFlag::Fragment<TestFlag::THROW_ON_FAILURE>      throw_on_failure;
+    typedef TestFlag::Fragment<TestFlag::WARNING_INTO_ERROR>    warning_into_error;
     typedef TestFlag::Fragment<TestFlag::PRINT_TIME>            print_time;
     typedef TestFlag::Fragment<TestFlag::SHOW_TESTS_LIST>       list_tests;
     typedef TestFlag::Fragment<TestFlag::SHOW_TESTS_LIST_WITH_WHERE> list_tests_with_where;
@@ -360,6 +361,11 @@ public:
     {
         get_vars().m_testpartresult_reporter = ptr;
     }
+    /** @private */
+    static bool has_output_option()
+    {
+        return !get_vars().m_output_option.empty();
+    }
 
 private:
     /**
@@ -410,7 +416,7 @@ private:
     */
     static void set_stream_result_to(const char* str)
     {
-        get_vars().m_stream_result_to = str == NULL ? "" : str;
+        get_vars().m_stream_result_to = detail::NullableString(str);
     }
 #endif
 
@@ -446,7 +452,7 @@ private:
     */
     static void set_output_option(const char* str)
     {
-        get_vars().m_output_option = str == NULL ? "" : str;
+        get_vars().m_output_option = detail::NullableString(str);
     }
 
     /**
@@ -454,7 +460,7 @@ private:
     */
     static void set_default_package_name(const char* str)
     {
-        get_vars().m_default_package_name = str == NULL ? "" : str;
+        get_vars().m_default_package_name = detail::NullableString(str);
     }
 
 private:
@@ -489,7 +495,7 @@ private:
         }
         const _Myt& operator = (const char* c_str_)
         {
-            m_option = c_str_ == NULL ? "" : c_str_;
+            m_option = detail::NullableString(c_str_);
             S(c_str_);
             return *this;
         }
@@ -695,12 +701,14 @@ private:
         return ParseCommandLineElemA(str);
     }
     static bool ParseCommandLineElemA(const char* str);
+    static bool ParseIutestOptionCommandLineElemA(const char* str);
+    static bool SetFlag(int enable, int mask = -1);
 
 private:
     /**
      * @brief   環境変数から設定構築
     */
-    static void LoadEnviromentVariable();
+    static void LoadEnvironmentVariable();
 
     /**
      * @brief   セットアップ
