@@ -2,7 +2,7 @@
 #
 # iuwandbox.py
 #
-# Copyright (C) 2014-2016, Takazumi Shirayanagi
+# Copyright (C) 2014-2017, Takazumi Shirayanagi
 # This software is released under the new BSD License,
 # see LICENSE
 #
@@ -11,11 +11,13 @@ import os
 import sys
 import re
 import codecs
+import argparse
 
 from time import sleep
 from argparse import ArgumentParser
 from wandbox import Wandbox
 from requests.exceptions import HTTPError
+from requests.exceptions import ConnectionError
 
 IUTEST_FUSED_SRC = os.path.normpath(os.path.join(os.path.dirname(__file__), '../../fused-src/iutest.min.hpp'))
 IUTEST_INCLUDE_PATH = os.path.normpath(os.path.join(os.path.dirname(__file__), '../../include'))
@@ -24,24 +26,32 @@ EXPAND_INCLUDE_REGEX = re.compile(r'^\s*#\s*include\s*"(.*?)"')
 IUTEST_INCG_REGEX = re.compile(r'\s*#\s*define[/\s]*(INCG_IRIS_\S*)\s*')
 
 iutest_incg_list = []
+workaround = True
+api_retries = 3
+api_retry_wait = 60
+
 
-#
 # command line option
 def parse_command_line():
+    global api_retries
+    global api_retry_wait
+
     parser = ArgumentParser()
     parser.add_argument(
         '-v',
         '--version',
         action='version',
-        version=u'%(prog)s version 4.2'
+        version=u'%(prog)s version 5.7'
     )
     parser.add_argument(
         '--list_compiler',
+        '--list-compiler',
         action='store_true',
         help='listup compiler.'
     )
     parser.add_argument(
         '--list_options',
+        '--list-options',
         metavar='COMPILER',
         help='listup compiler options.'
     )
@@ -59,13 +69,33 @@ def parse_command_line():
     parser.add_argument(
         '--default',
         action='store_true',
-        help='use default options.'
+        help='it is not work. default options are set by default (deprecated)'
+    )
+    parser.add_argument(
+        '--no-default',
+        action='store_true',
+        help='no set default options.'
+    )
+    parser.add_argument(
+        '--std',
+        metavar='VERSION',
+        help='set --std options.'
     )
     parser.add_argument(
         '--boost',
         metavar='VERSION',
         help='set boost options version X.XX or nothing.'
     )
+    parser.add_argument(
+        '--optimize',
+        action='store_true',
+        help='use optimization.'
+    )
+    parser.add_argument(
+        '--cpp-verbose',
+        action='store_true',
+        help='use cpp-verbose.'
+    )
     parser.add_argument(
         '--sprout',
         action='store_true',
@@ -83,6 +113,7 @@ def parse_command_line():
     parser.add_argument(
         '-f',
         '--compiler_option_raw',
+        '--compiler-option-raw',
         metavar='OPTIONS',
         action='append',
         default=['-D__WANDBOX__'],
@@ -91,6 +122,7 @@ def parse_command_line():
     parser.add_argument(
         '-r',
         '--runtime_option_raw',
+        '--runtime-option-raw',
         metavar='OPTIONS',
         action='append',
         help='runtime-time any additional options.'
@@ -133,11 +165,32 @@ def parse_command_line():
     )
     parser.add_argument(
         '--expand_include',
+        '--expand-include',
         action='store_true',
         help='expand include file.'
     )
+    parser.add_argument(
+        '--make',
+        action='store_true',
+        help=argparse.SUPPRESS
+    )
+    parser.add_argument(
+        '--retry-wait',
+        type=int,
+        default=api_retry_wait,
+        metavar='SECONDS',
+        help='Wait time for retry when HTTPError occurs'
+    )
+    parser.add_argument(
+        '--retry',
+        type=int,
+        default=api_retries,
+        metavar='COUNT',
+        help='Number of retries when HTTPError occurs'
+    )
     parser.add_argument(
         '--check_config',
+        '--check-config',
         action='store_true',
         help='check config.'
     )
@@ -158,6 +211,8 @@ def parse_command_line():
         help='source code file'
     )
     options = parser.parse_args()
+    api_retries = options.retry
+    api_retry_wait = options.retry_wait
     return options, parser
 
 
@@ -183,7 +238,6 @@ def make_include_filename(path, includes, included_files):
         return include_filename
 
 
-# 
 def is_iutest_included_file(filepath):
     if os.path.abspath(filepath).startswith(IUTEST_INCLUDE_PATH):
         incg = 'INCG_IRIS_' + os.path.basename(filepath).upper().replace('.', '_')
@@ -243,12 +297,34 @@ def make_code(path, encoding, expand, includes, included_files):
     return code
 
 
+def print_undefined_option(option_name, compiler):
+    print('Wandbox is not supported option [{0}] ({1})'.format(option_name, compiler))
+
+
 # check config
 def check_config(options):
+    has_error = False
     if not find_compiler(options.compiler):
         print('Wandbox is not supported compiler [' + options.compiler + ']')
         listup_compiler()
+        has_error = True
+    if options.options or options.std:
+        opt = get_options(options.compiler)
+        if options.options:
+            for o in options.options.split(','):
+                if o not in opt:
+                    print_undefined_option(o, options.compiler)
+                    has_error = True
+        if options.std:
+            if options.std not in opt:
+                print_undefined_option(options.std, options.compiler)
+                has_error = True
+        if has_error:
+            listup_options(options.compiler)
+    if has_error:
         sys.exit(1)
+    if options.default:
+        print('--default option is not work. default options are set by default (deprecated)')
 
 
 # setup additional files
@@ -257,68 +333,235 @@ def add_files(w, fileinfos):
         w.add_file(filename, code)
 
 
-# run wandbox
-def run_wandbox(code, includes, impliments, options):
-    w = Wandbox()
-    w.compiler(options.compiler)
+# create opt list
+def create_option_list(options):
+    def filterout_cppver(opt):
+        tmp = list(filter(lambda s: s.find('c++') == -1, opt))
+        tmp = list(filter(lambda s: s.find('gnu++') == -1, tmp))
+        return tmp
     opt = []
-    if options.options:
-        opt = options.options.split(',')
-    elif options.default:
+    if not options.no_default:
         opt = get_default_options(options.compiler)
+    if options.options:
+        for o in options.options.split(','):
+            if o not in opt:
+                if (o.find('c++') == 0) or (o.find('gnu++') == 0):
+                    opt = filterout_cppver(opt)
+                opt.append(o)
+    # std
+    if options.std:
+        opt = filterout_cppver(opt)
+        opt.append(options.std)
+    # optimize
+    if options.optimize and ('optimize' not in opt):
+        opt.append('optimize')
+    # cpp-verbose
+    if options.cpp_verbose and ('cpp-verbose' not in opt):
+        opt.append('cpp-verbose')
+    # boost
+    if workaround:
+        pass
+#        if options.compiler in ['clang-3.4', 'clang-3.3']:
+#            if not options.boost:
+#                options.boost = 'nothing'
     if options.boost:
+        if options.compiler not in options.boost:
+            options.boost = options.boost + '-' + options.compiler
         opt = list(filter(lambda s: s.find('boost') == -1, opt))
         opt.append('boost-' + str(options.boost))
-    if options.sprout and 'sprout' not in opt:
+    # sprout
+    if options.sprout and ('sprout' not in opt):
         opt.append('sprout')
-    if options.msgpack and 'msgpack' not in opt:
+    # msgpack
+    if options.msgpack and ('msgpack' not in opt):
         opt.append('msgpack')
-    w.options(','.join(opt))
-    if options.stdin:
-        w.stdin(options.stdin)
-    if options.compiler_option_raw:
-        co = '\n'.join(options.compiler_option_raw)
-        co = co.replace('\\n', '\n')
-        if options.compiler == "clang-3.5":
-            co += "\n-DIUTEST_HAS_HDR_CXXABI=0"
-        if options.compiler in ["clang-3.2", "clang-3.1", "clang-3.0"]:
-            co += "\n-Qunused-arguments"
-        w.compiler_options(co)
-    if options.runtime_option_raw:
-        ro = ''
-        for opt in options.runtime_option_raw:
-            ro += opt + '\n'
-        ro = ro.replace('\\n', '\n')
-        w.runtime_options(ro)
-    if options.save:
-        w.permanent_link(options.save)
-    for filename in impliments.keys():
-        w.add_compiler_options(filename)
-    if options.verbose:
-        w.dump()
-    w.code(code)
-    add_files(w, impliments)
-    add_files(w, includes)
+    return opt
+
+
+def expand_wandbox_options(w, compiler, options):
+    colist = []
+    defs = {}
+    for d in w.get_compiler_list():
+        if d['name'] == compiler:
+            if 'switches' in d:
+                switches = d['switches']
+                for s in switches:
+                    if ('name' in s) and ('display-flags' in s):
+                        defs[s['name']] = s['display-flags']
+                    elif 'options' in s:
+                        for o in s['options']:
+                            if ('name' in o) and ('display-flags' in o):
+                                defs[o['name']] = o['display-flags']
+    for opt in options:
+        if opt in defs:
+            colist.extend(defs[opt].split())
+    return colist
+
+
+def wandbox_api_call(callback, retries, retry_wait):
+    try:
+        return callback()
+    except (HTTPError, ConnectionError) as e:
+
+        def is_retry(e):
+            if not e.response:
+                return True
+            return e.response.status_code in [504]
+
+        if is_retry(e) and retries > 0:
+            try:
+                print(e.message)
+            except:
+                pass
+            print('wait {0}sec...'.format(retry_wait))
+            sleep(retry_wait)
+            return wandbox_api_call(callback, retries - 1, retry_wait)
+        else:
+            raise
+    except:
+        raise
+
+
+def wandbox_get_compilerlist():
+    return wandbox_api_call(Wandbox.GetCompilerList, api_retries, api_retry_wait)
+
+
+def run_wandbox_impl(w, options):
     if options.dryrun:
         sys.exit(0)
+    retries = options.retry
 
-    def run(retries):
-        try:
-            return w.run()
-        except HTTPError as e:
-            if e.response.status_code == 504 and retries > 0:
-                try:
-                    print(e.message)
-                except:
-                    pass
-                print("wait 30sec...")
-                sleep(30)
-                return run(retries - 1)
-            else:
-                raise
-        except:
-            raise
-    return run(3)
+    def run():
+        return w.run()
+    return wandbox_api_call(run, retries, options.retry_wait)
+
+
+def create_compiler_raw_option_list(options):
+    colist = []
+    if options.compiler_option_raw:
+        raw_options = options.compiler_option_raw
+        for x in raw_options:
+            colist.extend(re.split('\s(?=-)', x.strip('"')))
+    return colist
+
+
+# run wandbox (makefile)
+def run_wandbox_make(main_filepath, code, includes, impliments, options):
+    with Wandbox() as w:
+        w.compiler('bash')
+        woptions = create_option_list(options)
+        if options.stdin:
+            w.stdin(options.stdin)
+        impliments[os.path.basename(main_filepath)] = code
+
+        colist = create_compiler_raw_option_list(options)
+        colist.extend(expand_wandbox_options(w, options.compiler, woptions))
+
+        rolist = []
+        if options.runtime_option_raw:
+            for opt in options.runtime_option_raw:
+                rolist.extend(opt.split())
+
+        makefile = '#!/bin/make\n# generate makefile by iuwandbox.py\n'
+        makefile += '\nCXXFLAGS+='
+        for opt in colist:
+            makefile += opt + ' '
+        makefile += '\nOBJS='
+        for filename in impliments.keys():
+            makefile += os.path.splitext(filename)[0] + '.o '
+
+        makefile += '\n\
+prog: $(OBJS)\n\
+\t$(CXX) -o $@ $^ $(CXXFLAGS) $(LDFLAGS)\n\
+'
+
+        impliments['Makefile'] = makefile
+
+        bashscript = 'make -j 4\n'
+        bashscript += './prog '
+        for opt in rolist:
+            bashscript += opt + ' '
+        bashscript += '\n'
+        w.code(bashscript)
+
+        if options.save:
+            w.permanent_link(options.save)
+        if options.verbose:
+            w.dump()
+        add_files(w, impliments)
+        add_files(w, includes)
+
+        return run_wandbox_impl(w, options)
+
+
+# run wandbox (cxx)
+def run_wandbox_cxx(code, includes, impliments, options):
+    with Wandbox() as w:
+        w.compiler(options.compiler)
+        w.options(','.join(create_option_list(options)))
+        if options.stdin:
+            w.stdin(options.stdin)
+        colist = create_compiler_raw_option_list(options)
+
+        if workaround:
+            if options.compiler in ['clang-3.2']:
+                colist.append('-ftemplate-depth=1024')
+    #        if options.compiler in ['clang-3.4']:
+    #            colist.append('-DIUTEST_HAS_HDR_CXXABI=0')
+    #        if options.compiler in ['clang-3.3', 'clang-3.2', 'clang-3.1', 'clang-3.0']:
+    #            colist.append('-Qunused-arguments')
+    #        if options.compiler in ['clang-3.4', 'clang-3.3']:
+    #            colist.append('-fno-exceptions')
+    #            colist.append('-fno-rtti')
+            pass
+        if len(colist) > 0:
+            co = '\n'.join(colist)
+            co = co.replace('\\n', '\n')
+            w.compiler_options(co)
+        if options.runtime_option_raw:
+            rolist = []
+            for opt in options.runtime_option_raw:
+                rolist.extend(opt.split())
+            ro = '\n'.join(rolist)
+            ro = ro.replace('\\n', '\n')
+            w.runtime_options(ro)
+        if options.save:
+            w.permanent_link(options.save)
+        for filename in impliments.keys():
+            w.add_compiler_options(filename)
+        if options.verbose:
+            w.dump()
+        w.code(code)
+        add_files(w, impliments)
+        add_files(w, includes)
+
+        return run_wandbox_impl(w, options)
+
+
+# run wandbox
+def run_wandbox(main_filepath, code, includes, impliments, options):
+    if options.make:
+        return run_wandbox_make(main_filepath, code, includes, impliments, options)
+    else:
+        return run_wandbox_cxx(code, includes, impliments, options)
+
+
+def wandbox_hint(r):
+    if 'compiler_error' in r:
+        if 'undefined reference to `init_unit_test_suite' in r['compiler_error']:
+            print('hint:')
+            print('  If you do not use boost test, please specify the file with the main function first.')
+
+
+def text_transform(value):
+    try:
+        if isinstance(value, str):
+            return value.decode()
+        elif isinstance(value, unicode):
+            return value.encode('utf_8')
+    except:
+        pass
+    return value
 
 
 # show result
@@ -329,26 +572,28 @@ def show_result(r, options):
     if options.stderr:
         if 'compiler_output' in r:
             print('compiler_output:')
-            print(r['compiler_output'].encode('utf_8'))
+            print(text_transform(r['compiler_output']))
         if 'compiler_error' in r:
-            sys.stderr.write(r['compiler_error'].encode('utf_8'))
+            sys.stderr.write(text_transform(r['compiler_error']))
         if 'program_output' in r:
             print('program_output:')
-            print(r['program_output'].encode('utf_8'))
+            print(text_transform(r['program_output']))
         if options.xml is None and options.junit is None and 'program_error' in r:
-            sys.stderr.write(r['program_error'].encode('utf_8'))
+            sys.stderr.write(text_transform(r['program_error']))
     else:
         if 'compiler_message' in r:
             print('compiler_message:')
-            print(r['compiler_message'].encode('utf_8'))
+            print(text_transform(r['compiler_message']))
         if 'program_message' in r:
             print('program_message:')
-            print(r['program_message'].encode('utf_8'))
+            print(text_transform(r['program_message']))
     if 'url' in r:
         print('permlink: ' + r['permlink'])
         print('url: ' + r['url'])
     if 'signal' in r:
         print('signal: ' + r['signal'])
+    wandbox_hint(r)
+
     if 'status' in r:
         return int(r['status'])
     return 1
@@ -371,9 +616,9 @@ def show_parameter(r):
 def set_output_xml(options, t, xml):
     options.stderr = True
     if options.runtime_option_raw:
-        options.runtime_option_raw.append("--iutest_output=" + t + ":" + xml)
+        options.runtime_option_raw.append('--iutest_output=' + t + ':' + xml)
     else:
-        options.runtime_option_raw = ["--iutest_output=" + t + ":" + xml]
+        options.runtime_option_raw = ['--iutest_output=' + t + ':' + xml]
 
 
 def run(options):
@@ -384,7 +629,7 @@ def run(options):
     included_files = {}
     impliments = {}
     code = make_code(main_filepath, options.encoding, options.expand_include, includes, included_files)
-    
+
     for filepath_ in options.code[1:]:
         filepath = filepath_.strip()
         impliments[os.path.basename(filepath)] = make_code(filepath, options.encoding, options.expand_include, includes, included_files)
@@ -400,7 +645,7 @@ def run(options):
     if options.junit:
         xml = options.junit
         set_output_xml(options, 'junit', xml)
-    r = run_wandbox(code, includes, impliments, options)
+    r = run_wandbox(main_filepath, code, includes, impliments, options)
     b = show_result(r, options)
     if xml and 'program_error' in r:
         f = file_open(xml, 'w', options.encoding)
@@ -411,8 +656,7 @@ def run(options):
 
 # listup compiler
 def listup_compiler(verbose):
-    w = Wandbox()
-    r = w.get_compiler_list()
+    r = wandbox_get_compilerlist()
     for d in r:
         if d['language'] == 'C++':
             if verbose:
@@ -423,8 +667,7 @@ def listup_compiler(verbose):
 
 # find compiler
 def find_compiler(c):
-    w = Wandbox()
-    r = w.get_compiler_list()
+    r = wandbox_get_compilerlist()
     for d in r:
         if d['language'] == 'C++' and d['name'] == c:
             return True
@@ -433,10 +676,10 @@ def find_compiler(c):
 
 # listup options
 def listup_options(compiler):
-    w = Wandbox()
-    r = w.get_compiler_list()
+    r = wandbox_get_compilerlist()
     for d in r:
         if d['name'] == compiler:
+            print('# ' + compiler)
             if 'switches' in d:
                 switches = d['switches']
                 for s in switches:
@@ -451,10 +694,26 @@ def listup_options(compiler):
                             print('  ' + o['name'])
 
 
+def get_options(compiler):
+    r = wandbox_get_compilerlist()
+    opt = []
+    for d in r:
+        if d['name'] == compiler:
+            if 'switches' in d:
+                switches = d['switches']
+                for s in switches:
+                    if 'name' in s:
+                        opt.append(s['name'])
+                    elif 'options' in s:
+                        opt.append(s['default'])
+                        for o in s['options']:
+                            opt.append(o['name'])
+    return opt
+
+
 # get default options
 def get_default_options(compiler):
-    w = Wandbox()
-    r = w.get_compiler_list()
+    r = wandbox_get_compilerlist()
     opt = []
     for d in r:
         if d['name'] == compiler:
@@ -471,8 +730,7 @@ def get_default_options(compiler):
 
 # get permlink
 def get_permlink(options):
-    w = Wandbox()
-    r = w.get_permlink(options.permlink)
+    r = Wandbox.GetPermlink(options.permlink)
     p = r['parameter']
     show_parameter(p)
     print('result:')
@@ -500,5 +758,6 @@ def main():
             sys.exit(1)
         run(options)
 
+
 if __name__ == '__main__':
     main()
